import * as core from "@actions/core";
import { execSync } from "child_process";

interface Inputs {
  password: string;
  server: string;
  secrets: string[];
}

export async function run(): Promise<void> {
  try {
    const inputs = readInputs();
    core.info("🔑 Logging into Bitwarden CLI...");

    const sessionKey = loginToBitwarden(inputs.password, inputs.server);
    core.info("✅ Successfully logged in!");

    core.info("🔍 Retrieving secrets...");
    const parsedSecrets = retrieveSecrets(inputs.secrets, sessionKey);

    core.info("📄 Generating shell export statements...");
    generateShellExports(parsedSecrets);

    core.info("✅ Successfully retrieved and set secrets!");
  } catch (error) {
    core.setFailed(
      `❌ Error: ${error instanceof Error ? error.message : error}`
    );
  }
}

function readInputs(): Inputs {
  const password = core.getInput("password", { required: true });
  const server = core.getInput("server", { required: true });
  const secretsRaw = core.getInput("secrets", { required: true });

  const secrets = secretsRaw
    .split("\n")
    .map((s) => s.trim())
    .filter((s) => s.includes(">"));

  if (secrets.length === 0) {
    throw new Error("No valid secrets provided.");
  }

  return { password, server, secrets };
}

function loginToBitwarden(password: string, server: string): string {
  const output = execSync(`BW_SERVER=${server} bw login --raw`, {
    input: password + "\n",
    encoding: "utf-8",
  }).trim();

  return output;
}

function retrieveSecrets(
  secrets: string[],
  sessionKey: string
): Record<string, string> {
  const parsedSecrets: Record<string, string> = {};

  secrets.forEach((secretLine) => {
    const [id, name] = secretLine.split(">").map((s) => s.trim());
    try {
      const value = execSync(`BW_SESSION=${sessionKey} bw get secret ${id}`, {
        encoding: "utf-8",
      }).trim();
      parsedSecrets[name] = value;
    } catch {
      core.warning(`⚠️ Could not retrieve secret for ID: ${id}`);
    }
  });

  return parsedSecrets;
}

function generateShellExports(secrets: Record<string, string>): void {
  let exportCommands = "";
  for (const [key, value] of Object.entries(secrets)) {
    exportCommands += `export ${key}="${value}"\n`;
    core.setSecret(value);
  }

  console.log(exportCommands);
}

run();
